package org.opencdmp.filetransformer.rda.service.descriptiontemplatesearcher;

import org.opencdmp.commonmodels.models.descriptiotemplate.*;
import org.springframework.stereotype.Component;

import java.util.*;
import java.util.stream.Collectors;

@Component
public class TemplateFieldSearcherServiceImpl implements TemplateFieldSearcherService {

    @Override
    public Map<String, List<FieldModel>> searchFieldsById(DescriptionTemplateModel template, String value) {
        if (template == null || template.getDefinition() == null) return new HashMap<>();
        List<FieldModel> fieldModels = template.getDefinition().getFieldById(value);
        Map<String, List<FieldModel>> response = new HashMap<>();
        for (FieldModel fieldModel : fieldModels){
            String key = this.buildFieldKey(template, fieldModel);
            if (!response.containsKey(key)) response.put(key, new ArrayList<>());
            response.get(key).add(fieldModel);
        }
        return response;
    }

    @Override
    public Map<String, List<FieldModel>> searchFieldsBySemantics(DescriptionTemplateModel template, String value) {
        if (template == null || template.getDefinition() == null) return new HashMap<>();
        List<FieldModel> fieldModels = template.getDefinition().getAllField();
        if (fieldModels == null) return new HashMap<>();
        fieldModels = fieldModels.stream().filter(x-> x.getSchematics() != null && x.getSchematics().contains(value)).toList();
        Map<String, List<FieldModel>> response = new HashMap<>();
        for (FieldModel fieldModel : fieldModels){
            String key = this.buildFieldKey(template, fieldModel);
            if (!response.containsKey(key)) response.put(key, new ArrayList<>());
            response.get(key).add(fieldModel);
        }
        return response;
    }
    
    private String buildFieldKey(DescriptionTemplateModel template, FieldModel fieldModel){
        if (template == null || template.getDefinition() == null || template.getDefinition().getPages() == null) return null;
        for (PageModel pageModel : template.getDefinition().getPages().stream().sorted(Comparator.comparing(PageModel::getOrdinal)).toList()){
            String key = this.buildFieldKey(pageModel, fieldModel);
            if (key != null) return pageModel.getOrdinal() + "." + key;
        }
        return null;
    }
    private String buildFieldKey(PageModel pageModel, FieldModel fieldModel){
        if (pageModel == null || pageModel.getSections() == null) return null;
        for (SectionModel sectionModel : pageModel.getSections().stream().sorted(Comparator.comparing(SectionModel::getOrdinal)).toList()){
            String key = this.buildFieldKey(sectionModel, fieldModel);
            if (key != null) return sectionModel.getOrdinal() + "." + key;
        }
        return null;
    }

    private String buildFieldKey(SectionModel sectionModel, FieldModel fieldModel){
        if (sectionModel == null) return null;
        if (sectionModel.getSections() != null) {
            for (SectionModel innerSectionModel : sectionModel.getSections().stream().sorted(Comparator.comparing(SectionModel::getOrdinal)).toList()) {
                String key = this.buildFieldKey(innerSectionModel, fieldModel);
                if (key != null) return innerSectionModel.getOrdinal() + "." + key;
            }
        }
        if (sectionModel.getFieldSets() != null) {
            for (FieldSetModel fieldSetModel : sectionModel.getFieldSets().stream().sorted(Comparator.comparing(FieldSetModel::getOrdinal)).toList()) {
                String key = this.buildFieldKey(fieldSetModel, fieldModel);
                if (key != null) return fieldSetModel.getOrdinal() + "." + key;

            }
        }
        return null;
    }

    private String buildFieldKey(FieldSetModel fieldSetModel, FieldModel fieldModel){
        if (fieldSetModel == null || fieldSetModel.getFields() == null) return null;
        for (FieldModel currentFieldModel : fieldSetModel.getFields().stream().sorted(Comparator.comparing(FieldModel::getOrdinal)).toList()){
            if (currentFieldModel.getId().equals(fieldModel.getId())) return currentFieldModel.getOrdinal().toString();
        }
        return null;
    }
}
